import { computed, defineComponent, h, mergeProps, onMounted, reactive, ref, toRefs, useModel, watchEffect } from "vue";
import { reactiveOmit, useComposedRefs, useControllable, useForwardRef, useListeners } from "@oku-ui/use-composable";
import { composeEventHandlers } from "@oku-ui/utils";
import { Primitive } from "@oku-ui/primitive";
import { OkuBubbleInput } from "./bubble-input.mjs";
import { getState, isIndeterminate } from "./utils.mjs";
import { CHECKBOX_NAME, checkboxProps, checkboxProvider, scopeCheckboxProps } from "./props.mjs";
const Checkbox = defineComponent({
  name: CHECKBOX_NAME,
  components: {
    OkuBubbleInput
  },
  inheritAttrs: false,
  props: {
    ...checkboxProps.props,
    ...scopeCheckboxProps
  },
  emits: checkboxProps.emits,
  setup(props, { attrs, emit, slots }) {
    const {
      scopeOkuCheckbox,
      name,
      checked: checkedProp,
      defaultChecked,
      required,
      disabled,
      value,
      ...checkboxProps2
    } = toRefs(props);
    const _reactive = reactive(checkboxProps2);
    const otherProps = reactiveOmit(_reactive, (key, _value) => key === void 0);
    const forwardedRef = useForwardRef();
    const emits = useListeners();
    const buttonRef = ref(null);
    const composedRefs = useComposedRefs(forwardedRef, buttonRef);
    const hasConsumerStoppedPropagationRef = ref(false);
    const isFormControl = ref(false);
    onMounted(() => {
      isFormControl.value = buttonRef.value ? typeof buttonRef.value.closest === "function" && Boolean(buttonRef.value.closest("form")) : true;
    });
    const modelValue = useModel(props, "modelValue");
    const proxyChecked = computed({
      get: () => modelValue.value !== void 0 ? modelValue.value : checkedProp.value !== void 0 ? checkedProp.value : void 0,
      set: () => {
      }
    });
    const { state, updateValue } = useControllable({
      prop: computed(() => proxyChecked.value),
      defaultProp: computed(() => defaultChecked.value),
      onChange: (result) => {
        modelValue.value = result;
        emit("checkedChange", result);
        emit("update:modelValue", result);
      },
      initialValue: false
    });
    const initialCheckedStateRef = ref(state.value);
    watchEffect((onInvalidate) => {
      const form = buttonRef.value?.form;
      if (form) {
        const reset = () => updateValue(initialCheckedStateRef.value);
        form.addEventListener("reset", reset);
        onInvalidate(() => form.removeEventListener("reset", reset));
      }
    });
    checkboxProvider({
      scope: scopeOkuCheckbox.value,
      state,
      disabled
    });
    return () => [
      h(Primitive.button, {
        "type": "button",
        "role": "checkbox",
        "aria-checked": computed(() => isIndeterminate(state.value) ? "mixed" : state.value).value,
        "aria-required": required.value,
        "data-state": computed(() => getState(state.value)).value,
        "data-disabled": disabled.value ? "" : void 0,
        "disabled": disabled.value,
        "value": value.value,
        ...mergeProps(attrs, otherProps, emits),
        "ref": composedRefs,
        "onKeydown": composeEventHandlers((event) => {
          emit("keydown", event);
        }, (event) => {
          if (event.key === "Enter")
            event.preventDefault();
        }),
        "onClick": composeEventHandlers((event) => {
          emit("click", event);
        }, (event) => {
          updateValue(isIndeterminate(state.value) ? true : !state.value);
          if (isFormControl.value) {
            if (!hasConsumerStoppedPropagationRef.value)
              event.stopPropagation();
          }
        })
      }, () => slots.default?.()),
      isFormControl.value && h(OkuBubbleInput, {
        control: buttonRef.value,
        bubbles: computed(() => !hasConsumerStoppedPropagationRef.value).value,
        name: name.value,
        value: value.value,
        checked: state.value,
        required: required.value,
        disabled: disabled.value,
        // We transform because the input is absolutely positioned but we have
        // rendered it **after** the button. This pulls it back to sit on top
        // of the button.
        style: { transform: "translateX(-100%)" }
      })
    ];
  }
});
export const OkuCheckbox = Checkbox;
